﻿using System;
using System.Collections.Generic;
using System.Data.Entity;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GameStore.Entities.Configuration;
using GameStore.Entities.Contexts;
using GameStore.Entities.DataAccessModels;
using Microsoft.Extensions.Options;

namespace GameStore.Entities.Repositories
{
    /// <summary>
    /// Exposes create and read operations for <see cref="Developer"/> entity without exposing entity framework.
    /// </summary>
    public class GameDevelopersRepository : IGameDevelopersRepository
    {
        private GameStoreContext _context;

        public GameDevelopersRepository(IOptions<ConnectionOptions> connectionOptions)
        {
            _context = new GameStoreContext(connectionOptions.Value.ConnectionString);
        }

        public async Task<Developer> GetSingleDeveloper(string codeName) 
            => await _context
                .Developers
                .Where(developer => developer.CodeName == codeName)
                .SingleOrDefaultAsync();

        public async Task<Developer> AddDeveloper(Developer developer)
        {
            if (developer == null)
                throw new ArgumentNullException(nameof(developer));

            var addedDeveloper = _context.Developers.Add(developer);
            await _context.SaveChangesAsync();

            return addedDeveloper;
        }

        public async Task<Developer[]> GetAllDevelopers()
            => await _context.Developers.ToArrayAsync();
        
    }
}
