using System.Threading.Tasks;
using Microsoft.Extensions.Logging;
using GameStore.Entities.Repositories;
using System.Linq;
using System;
using AutoMapper;
using GameStore.Entities.DataAccessModels;
using System.Collections.Generic;

namespace GameStore.Services.GameStock
{
    public class GameStockService : IGameStockService
    {
        private readonly ILogger<IGameStockService> _logger;
        private readonly IGameStoreRepository _storeRepository;
        private IGameDevelopersRepository _developersRepository;
        private IMapper _mapper;

        public GameStockService(
            IGameStoreRepository storeRepository, 
            IGameDevelopersRepository developersRepository, 
            ILogger<IGameStockService> logger,
            IMapper mapper)
        {
            _storeRepository = storeRepository;
            _developersRepository = developersRepository;
            _logger = logger;
            _mapper = mapper;
        }

        public async Task<DataTransferModels.GameItem[]> GetAllGames()
        {
            _logger.LogInformation("Starting Game service method");

            IEnumerable<Game> dataAccessGamesModel = await _storeRepository.GetAllGames();
            return _mapper
                .Map<IEnumerable<Game>, IEnumerable<DataTransferModels.GameItem>>(dataAccessGamesModel)
                .ToArray();
        }

        public async Task<DataTransferModels.Developer[]> GetAllDevelopers()
        {
            _logger.LogInformation("Starting Game service method");

            var dataAccessDeveloperModel = await _developersRepository.GetAllDevelopers();
            return _mapper.Map<DataTransferModels.Developer[]>(dataAccessDeveloperModel);
        }

        public async Task<DataTransferModels.Game> GetGame(string codeName)
        {
            Game dataAccessGameModel = await _storeRepository.GetSingleGame(codeName);

            return _mapper.Map<DataTransferModels.Game>(dataAccessGameModel);
        }

        public async Task<DataTransferModels.Game> AddGame(DataTransferModels.GameItem game, string developerCodeName)
        {
            var developer = await _developersRepository.GetSingleDeveloper(developerCodeName);
            if (developer == null)
            {
                var message = $"Deloper with code name {developerCodeName} not found.";
                _logger.LogError(message);
                throw new InvalidOperationException(message);
            }

            Game dataAccessGameModel = _mapper.Map<Game>(game);
            dataAccessGameModel = await _storeRepository.AddGame(dataAccessGameModel, developer, Enumerable.Empty<Award>());

            return _mapper.Map<DataTransferModels.Game>(dataAccessGameModel);
        }

        public async Task<DataTransferModels.Developer> AddDeveloper(string codeName, string displayName)
        {
            var existingDeveloper = _developersRepository.GetSingleDeveloper(codeName);
            if(existingDeveloper != null)
            {
                var message = $"Deloper with code name {codeName} already exists.";
                _logger.LogError(message);
                throw new InvalidOperationException(message);
            }

            Developer dataAccessDeveloperModel = await _developersRepository.AddDeveloper(new Developer
            {
                CodeName = codeName,
                DisplayName = displayName
            });
            return _mapper.Map<DataTransferModels.Developer>(dataAccessDeveloperModel);
        }
    }
}