using System.Collections.Generic;
using System.Threading.Tasks;
using Microsoft.AspNetCore.Mvc;
using GameStore.Models;
using GameStore.Services.GameStock;
using AutoMapper;
using Microsoft.AspNetCore.Mvc.Rendering;
using System.Linq;

namespace GameStore.Controllers.Web
{
    public class GamesController : Controller
    {
        private readonly IGameStockService _gameStockService;
        private IMapper _mapper;

        public GamesController(IGameStockService gameStockService, IMapper mapper)
        {
            _gameStockService = gameStockService;
            _mapper = mapper;
        }

        // GET: Games
        [ActionName("Index")]
        public async Task<IActionResult> List()
        {
            var gamesDTO = await _gameStockService.GetAllGames();
            return View(_mapper.Map<IEnumerable<GameViewModel>>(gamesDTO));
        }

        // GET: Games/Details/[codeName]
        public async Task<IActionResult> Details(string codename)
        {
            if (codename == null)
            {
                return NotFound();
            }

            var gameDTO = await _gameStockService.GetGame(codename);
            if (gameDTO == null)
            {
                return NotFound();
            }

            return View(_mapper.Map<GameDetailsViewModel>(gameDTO));
        }

        // GET: Games/Create
        public async Task<IActionResult> Create()
        {
            var newGame = new CreateGameViewModel();
            var developers = await _gameStockService.GetAllDevelopers();
            var developersList = developers.Select(d => new { Id = d.DeveloperCodeName, Value = d.DisplayName });
            newGame.DeveloperList = new SelectList(developersList, "Id", "Value");

            return View(newGame);
        }

        // POST: Games/Create
        // To protect from overposting attacks, please enable the specific properties you want to bind to, for 
        // more details see http://go.microsoft.com/fwlink/?LinkId=317598.
        [HttpPost]
        [ValidateAntiForgeryToken]
        public async Task<IActionResult> Create([Bind("Name", "CodeName", "Price", "DeveloperCodeName")]CreateGameViewModel game)
        {
            if (ModelState.IsValid)
            {
                await _gameStockService.AddGame(_mapper.Map<Services.DataTransferModels.GameItem>(game), game.DeveloperCodeName);
                return RedirectToAction("Index");
            }

            var developers = await _gameStockService.GetAllDevelopers();
            game.DeveloperList = new SelectList(developers, "DeveloperCodeName", "DisplayName");
            return View(game);
        }
    }
}
