function Dispatcher() {
    this._stores = [];
}

Dispatcher.prototype.register = function (store) {
    if (!store || !store.update) {
        throw new Error('You should provide a store that has an `update` method.');
    }

    const consumers = [];

    const changed = () => consumers.forEach(consumer => consumer(store));

    const subscribe = consumer => {
        consumers.push(consumer);
        consumer(store);
    };

    this._stores.push({
        store: store,
        changed: changed
    });

    return subscribe;
};

Dispatcher.prototype.dispatch = function (action) {
    console.log(`Dispatch '${action.type}'`, action.payload);

    this._stores.forEach(entry => {
        entry.store.update(action, entry.changed)
    });
};

// Export singleton instance
export default (new Dispatcher())
