import actionTypes from './actionTypes';

export function createGameStore(initialGames) {
    const games = initialGames || [];

    return {
        _state: {
            games: games,
            editedGame: null,
        },

        getGames: function () {
            return this._state.games;
        },

        getEditedGame: function () {
            return this._state.editedGame;
        },

        update: function (action, changed) {
            switch(action.type) {
              case actionTypes.ShowDetailsFor:
                // Object assign will make a shallow copy of the object.
                // We can modify editedGame as we want.
                this._state.editedGame = Object.assign({}, action.payload.game);
                changed();
                break;

              case actionTypes.CancelEdit:
                this._state.editedGame = null;
                changed();
                break;

              case actionTypes.UpdateEditedGame:
                // TODO: add validation for edited game
                const editedGame = this._state.editedGame;
                this._state.games = this._state.games.map(game => {
                  if (game.gameCodeName === editedGame.gameCodeName) {
                    return editedGame;
                  } else {
                    return game;
                  }
                });
                this._state.editedGame = null;
                changed();
                break;

              case actionTypes.UpdateEditedGameName:
                // TODO: add minlength/maxlength validation
                this._state.editedGame.displayName = action.payload.updatedName;
                changed();
                break;

              case actionTypes.UpdateEditedGamePrice:
                // use parseInto because input sends a string number
                this._state.editedGame.gamePrice = parseInt(action.payload.updatedPrice);
                changed();
                break;

              default:
                break;
            }
        }
    }
}
