﻿using System.Threading.Tasks;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.AspNetCore.Mvc.Infrastructure;
using Microsoft.AspNetCore.Mvc.Routing;
using Microsoft.Extensions.Logging;
using GameStore.Controllers.Api;
using GameStore.Services.DataTransferModels;
using GameStore.Services.GameStock;
using GameStore.Services.RandomId;
using NSubstitute;
using Xunit;

namespace GameStore.UnitTests.Api
{
    public class GamesControllerTests
    {
        [Fact]
        public async Task GetsOnlyIdAndPrice()
        {
            // Arrange
            var serviceMock = Substitute.For<IGameStockService>();
            var randomIdMock = Substitute.For<IRandomIdService>();
            var actionContextAccessorMock = Substitute.For<IActionContextAccessor>();
            var httpContextAccessorMock = Substitute.For<IHttpContextAccessor>();
            var urlHelperFactoryMock = Substitute.For<IUrlHelperFactory>();
            var loggerMock = Substitute.For<ILogger<GamesController>>();

            serviceMock.GetAllGames().ReturnsForAnyArgs(new[]
            {
                new GameItem
                {
                    GameCodeName = "callofduty",
                    GamePrice = 99m,
                    DisplayName = "call of duty"
                },
                new GameItem
                {
                    GameCodeName = "wow",
                    GamePrice = 99m,
                    DisplayName = "world of warcraft"
                },
            });

            // Act
            var controller = new GamesController(serviceMock, randomIdMock, actionContextAccessorMock, httpContextAccessorMock, urlHelperFactoryMock, loggerMock);
            var result = await controller.GetAllGames(0, new[] {"GameCodeName", "GamePrice"});

            // Assert
            Assert.IsType<OkObjectResult>(result);

            var games = (GameItem[]) ((OkObjectResult) result).Value;

            Assert.NotEmpty(games);
            foreach (var game in games)
            {
                Assert.NotNull(game.GameCodeName);
                Assert.NotNull(game.GamePrice);
            }
        }
    }
}
