﻿using System.Collections.Generic;
using System.Reflection;
using AutoMapper;
using GameStore.Entities.Configuration;
using GameStore.Entities.DataAccessModels;
using Xunit;

namespace GameStore.Entities.UnitTests
{
    public class MapperConfigurationTests
    {
        [Fact]
        public void AssertMapperConfiguration()
        {
            // Ensures no properties are missed when mapping from narrow to wide type
            GetMapperConfiguration().AssertConfigurationIsValid();
        }


        [Fact]
        public void Map_Game_GameWrapper()
        {
            // Ensures mapping from Game to GameWrapper is present
            var game = new Game
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Id = new System.Guid("{32C76288-88B6-4B09-BBA4-50267AD9D979}"),
                Price = 123.7m,
            };
            var expectedGameWrapper = new GameWrapper
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Id = new System.Guid("{32C76288-88B6-4B09-BBA4-50267AD9D979}"),
                Price = 123.7m,
                Awards = new List<Award>(),
                Developer = null
            };

            var actualGameWrapper = GetMapper().Map<Game, GameWrapper>(game);

            AssertPropertyEquals(expectedGameWrapper, actualGameWrapper);
        }


        [Fact]
        public void Map_GameWrapper_Game()
        {
            // Ensures mapping from GameWrapper to Game is present
            var gameWrapper = new GameWrapper
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Id = new System.Guid("{32C76288-88B6-4B09-BBA4-50267AD9D979}"),
                Price = 123.7m,
                Awards = {
                    new Award {
                        Id = new System.Guid("{32C76288-0000-4B09-1111-50267AD9D979}"),
                        CodeName = "Award code name",
                        DisplayName = "Award display"
                    }
                },
                Developer = null
            };
            var expectedGame = new Game
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Id = new System.Guid("{32C76288-88B6-4B09-BBA4-50267AD9D979}"),
                Price = 123.7m,
            };

            var actualGame = GetMapper().Map<GameWrapper, Game>(gameWrapper);

            AssertPropertyEquals(expectedGame, actualGame);
        }


        /// <summary>
        /// Expects equality of (presumably scaler) values of each public property of <typeparamref name="TType"/>.
        /// </summary>
        private static void AssertPropertyEquals<TType>(TType expectedDataAccessGameModel, TType actualDataAccessGameModel)
        {
            foreach (var property in typeof(TType).GetProperties(BindingFlags.Public | BindingFlags.GetProperty))
            {
                Assert.Equal(property.GetValue(expectedDataAccessGameModel), property.GetValue(actualDataAccessGameModel));
            }
        }

        private static MapperConfiguration GetMapperConfiguration() => new MapperConfiguration(EntitiesMapperConfiguration.InitialializeMappings);

        private static IMapper GetMapper() => GetMapperConfiguration().CreateMapper();
    }
}
