﻿using System;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using GameStore.Services.Configuration;
using GameStore.Services.GameStock;
using GameStore.Services.RandomId;
using NSubstitute;
using Xunit;
using GameStore.Entities.Repositories;
using System.Threading.Tasks;
using AutoMapper;

namespace GameStore.UnitTests
{
    public class GameStockServiceTests
    {

        [Theory]
        [InlineData("wow")]
        [InlineData("WoW")]
        [InlineData("wOw")]
        public void GetsGameByCodename(string codename)
        {
            // Arrange
            var randomIdMock = Substitute.For<IRandomIdService>();
            var storeRepositoryMock = Substitute.For<IGameStoreRepository>();
            var developersRepositoryMock = Substitute.For<IGameDevelopersRepository>();
            var loggerMock = Substitute.For<ILogger<IGameStockService>>();
            var mapper = new MapperConfiguration(ServicesMapperConfiguration.InitialializeMappings).CreateMapper();

            storeRepositoryMock.GetSingleGame(codename).Returns(Task.FromResult(new Entities.DataAccessModels.GameWrapper
            {
                CodeName = "WOW",
            }));


            // Act
            var service = new GameStockService(storeRepositoryMock, developersRepositoryMock, loggerMock, mapper);
            var result = service.GetGame(codename).Result;

            // Assert
            Assert.NotNull(result);
            Assert.Equal(result.GameCodeName, codename, StringComparer.CurrentCultureIgnoreCase);
        }
    }
}
