﻿using System.Reflection;
using AutoMapper;
using GameStore.Entities.DataAccessModels;
using GameStore.Services.Configuration;
using Xunit;

namespace GameStore.Services.UnitTests
{
    public class MapperConfigurationTests
    {
        [Fact]
        public void AssertMapperConfiguration()
        {
            // Ensures no properties are missed when mapping from narrow to wide type
            GetMapperConfiguration().AssertConfigurationIsValid();
        }

        [Fact]
        public void Map_ServiceGame_DatabaseGame()
        {
            // Ensures mapping from DTO game to DAO game is present
            var dataTransportGameModel = new DataTransferModels.GameItem
            {
                DisplayName = "Display Name",
                GameCodeName = "CodeName",
                GamePrice = 125.3m,
            };
            var expectedDataAccessGameModel = new Game
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Price = 125.3m,
            };

            var actualDataAccessGameModel = GetMapper().Map<DataTransferModels.GameItem, Game>(dataTransportGameModel);

            AssertPropertyEquals(expectedDataAccessGameModel, actualDataAccessGameModel);
        }

        [Fact]
        public void Map_DatabaseGame_ServiceGame()
        {
            // Ensures mapping from DAO game to DTO game is present
            var dataAccessGameModel = new Game
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
                Price = 125.3m,
            };
            var expectedDataTransportGameModel = new DataTransferModels.GameItem
            {
                DisplayName = "Display Name",
                GameCodeName = "CodeName",
                GamePrice = 125.3m,
            };

            var actualDataTransportGameModel = GetMapper().Map<Game, DataTransferModels.GameItem>(dataAccessGameModel);

            AssertPropertyEquals(expectedDataTransportGameModel, actualDataTransportGameModel);
        }

        [Fact]
        public void Map_DatabaseDeveloper_ServiceDeveloper()
        {
            // Ensures mapping from DAO developer to DTO developer is present
            var dataAccessGameModel = new Developer
            {
                DisplayName = "Display Name",
                CodeName = "CodeName",
            };
            var expectedDataTransportGameModel = new DataTransferModels.Developer
            {
                DisplayName = "Display Name",
                DeveloperCodeName = "CodeName",
            };

            var actualDataTransportGameModel = GetMapper().Map<Developer, DataTransferModels.Developer>(dataAccessGameModel);

            AssertPropertyEquals(expectedDataTransportGameModel, actualDataTransportGameModel);
        }

        /// <summary>
        /// Expects equality of (presumably scaler) values of each public property of <typeparamref name="TType"/>.
        /// </summary>
        private static void AssertPropertyEquals<TType>(TType expectedDataAccessGameModel, TType actualDataAccessGameModel)
        {
            foreach (var property in typeof(TType).GetProperties(BindingFlags.Public | BindingFlags.GetProperty))
            {
                Assert.Equal(property.GetValue(expectedDataAccessGameModel), property.GetValue(actualDataAccessGameModel));
            }
        }

        private static MapperConfiguration GetMapperConfiguration() => new MapperConfiguration(ServicesMapperConfiguration.InitialializeMappings);

        private static IMapper GetMapper() => GetMapperConfiguration().CreateMapper();
    }
}
