namespace GameStore.Migrations.GameStoreContextMigrations
{
    using System;
    using System.Data.Entity;
    using System.Data.Entity.Migrations;
    using System.Linq;
    using Entities.DataAccessModels;

    internal sealed class GameStoreContextConfiguration : DbMigrationsConfiguration<GameStore.Entities.Contexts.GameStoreContext>
    {
        public GameStoreContextConfiguration()
        {
            AutomaticMigrationsEnabled = true;
            AutomaticMigrationDataLossAllowed = true;
            MigrationsDirectory = @"GameStoreContextMigrations\Migrations";
        }

        /// <summary>
        /// When using <see cref="IDbSet{TEntity}.AddOrUpdate()"/>, always specify pattern to match between new and existing object.
        /// Do no use this in production code to avoid unitentional data loss.
        /// </summary>
        protected override void Seed(GameStore.Entities.Contexts.GameStoreContext context)
        {
            var developer = new Developer
            {
                CodeName = "AB",
                DisplayName = "Activisioned Blizard"
            };

            var developer2 = new Developer
            {
                CodeName = "Atari",
                DisplayName = "Atari"
            };

            context
                .Developers
                .AddOrUpdate(
                    dev => dev.CodeName,
                    developer, developer2);
            context.SaveChanges();

            context.Games.AddOrUpdate(
                game => game.DisplayName,
                new GameWrapper
                {
                    CodeName = "CoD",
                    Price = 0.009m,
                    DisplayName = "Call of Duty (free)",
                    Developer = developer
                },
                new GameWrapper
                {
                    CodeName = "WoW",
                    Price = 99m,
                    DisplayName = "World of Warcraft and Wizardry",
                    Developer = developer
                });
        }
    }
}
