﻿using System.Data.Entity;
using GameStore.Entities.Contexts;
using GameStore.Migrations.GameStoreContextMigrations;
using Migrations.GameContextMigrations;

using static System.Console;

namespace GameStore.Migrations
{
    class Program
    {
        private const string CONNECTION_STRING_NAME = "GameStoreConnection";

        static void Main(string[] args)
        {
            WriteLine("Setting migration initializer...");

            // Ensure migration initializer is used and it uses context's connection
            Database.SetInitializer(new MigrateDatabaseToLatestVersion<GameStoreContext, GameStoreContextConfiguration>(useSuppliedContext: true));
            Database.SetInitializer(new MigrateDatabaseToLatestVersion<GamersContext, GamersContextConfiguration>(useSuppliedContext: true));

            // Initialize databases of both contexts with same connection string
            using (var db = new GameStoreContext(CONNECTION_STRING_NAME))
            {
                IntializeDatabase(db);
            }

            using (var db = new GamersContext(CONNECTION_STRING_NAME))
            {
                IntializeDatabase(db);
            }

            WriteLine("Everything is gonna be OK...");
            ReadKey();
        }
        
        /// <summary>
        /// Forces database initialization on provided <paramref name="context"/>.
        /// </summary>
        /// <typeparam name="TContext">Type of concrete implementation of <see cref="DbContext"/>.</typeparam>
        /// <param name="context">Instance of concrete context</param>
        private static void IntializeDatabase<TContext>(TContext context)
            where TContext : DbContext
        {
            WriteLine($"Initializing {typeof(TContext).FullName}...");
            context.Database.Initialize(force: true);
        }
    }
}
